# -*- coding: utf-8 -*-

#change the current directory
import os
os.chdir("... your directory ...")

#import the Pandas library - http://pandas.pydata.org/
#it provides - among other - tools for the data handling
import pandas
pima = pandas.read_table("pima.txt",sep="\t",header=0)

#number of rows and columns
print(pima.shape)

#columns name
print(pima.columns)

#data type for each column
print(pima.dtypes)

#transform the data into a NumPy matrix
data = pima.as_matrix()

#### HOLD OUT framework ####

#X matrix for the descriptors
X = data[:,0:8]

#y vector for the target attribute
y = data[:,8]

#model_selection module
from sklearn import model_selection

#splitting the dataset - test = 300, train (app) = 768-test = 468
X_app,X_test,y_app,y_test = model_selection.train_test_split(X,y,test_size = 300,random_state=0)
print(X_app.shape,X_test.shape,y_app.shape,y_test.shape)


#from the linear_model module of the scikit-learn package (sklearn)
#import the LogisticRegression class
from sklearn.linear_model import LogisticRegression

#instantiate the object lr
lr = LogisticRegression()

#launch the learning process
#i.e. fit the model to the labelled training set
modele = lr.fit(X_app,y_app)

#the outputs are lacking
#the coefficients and the intercept
print(modele.coef_,modele.intercept_)

#prediction on the test sample
y_pred = modele.predict(X_test)

#import the metrics module
#quantify the quality of the prediction
from sklearn import metrics

#confusion matrix
cm = metrics.confusion_matrix(y_test,y_pred)
print(cm)

#success rate
acc = metrics.accuracy_score(y_test,y_pred)
print(acc)

#error rate
err = 1.0 - acc
print(err)

#sensibility
se = metrics.recall_score(y_test,y_pred,pos_label='positive')
print(se)

#write our own performance metric e.g. specificity
def specificity(y,y_hat):
    #confusion matrix
    mc = metrics.confusion_matrix(y,y_hat)
    #negative corresponds to the first row of the matrix
    import numpy
    res = mc[0,0]/numpy.sum(mc[0,:])
    #return the specificity
    return res
#

#make the function usable as a scorer object
specificite = metrics.make_scorer(specificity,greater_is_better=True)

#use the new scorer object
sp = specificite(modele,X_test,y_test)
print(sp)

#### CROSS VALIDATION ####

#import the LogisticRegression class
from sklearn.linear_model import LogisticRegression

#instantiate and initialize the object
lr = LogisticRegression()

#fit the model on the whole dataset (X,y)
modele_all = lr.fit(X,y)

#print the coefficients and the intercept
print(modele_all.coef_,modele_all.intercept_)

#import the cross_validation module
from sklearn import cross_validation

#10-fold cross validation
succes = cross_validation.cross_val_score(lr,X,y,cv=10,scoring='accuracy')

#details of the results (result for each fold)
print(succes)

#CV success rate estimation
print(succes.mean())


#### SCORING ####

#Logistic Regression class
from sklearn.linear_model import LogisticRegression

#instantiate and initialize
lr = LogisticRegression()

#fit to the learning set
modele = lr.fit(X_app,y_app)

#estimate the posterior probabilities for the test set
probas = lr.predict_proba(X_test)

#score of the 'presence' (index = 1)
score = probas[:,1]

#transf. into 0/1 (dummy variable) of Y_test
pos = pandas.get_dummies(y_test).as_matrix()

#column for the "positive" class
pos = pos[:,1]

#number of positive instances into the test set
import numpy
npos = numpy.sum(pos)

#indices that would sort according to the score
index = numpy.argsort(score)

#invert the indices
index = index[::-1]

#sort the positive class membership (0/1)
sort_pos = pos[index]

#cumulative sum
cpos = numpy.cumsum(sort_pos)

#recall
rappel = cpos/npos

#number of instances in the test set
n = y_test.shape[0]

#target size
taille = numpy.arange(start=1,stop=301,step=1)

#target size in percentage
taille = taille / n

#graphical representation of the gains chart
import matplotlib.pyplot as plt
plt.title('Courbe de gain')
plt.xlabel('Taille de cible')
plt.ylabel('Rappel')
plt.xlim(0,1)
plt.ylim(0,1)
plt.scatter(taille,taille,marker='.',color='blue')
plt.scatter(taille,rappel,marker='.',color='red')
plt.show()


#### GRID SEARCH ####

#svm (support vector machine)
from sklearn import svm

#default settings : kernel = RBF and C = 1.0
mvs = svm.SVC()

#fit to the training set
modele2 = mvs.fit(X_app,y_app)

#prediction on the test set
y_pred2 = modele2.predict(X_test)

#confusion matrix
print(metrics.confusion_matrix(y_test,y_pred2))

#succes rate
print(metrics.accuracy_score(y_test,y_pred2))

#combinations of parameters to evaluate
parametres = [{'C':[0.1,1,10],'kernel':['rbf','linear']}]

#import the class
from sklearn import model_selection

#measuring the success rate (accuracy rate)
#for the search of the best parameters
grid = model_selection.GridSearchCV(estimator=mvs,param_grid=parametres,scoring='accuracy')

#launch the search
grille = grid.fit(X_app,y_app)

#results for each combination
print(pandas.DataFrame.from_dict(grille.cv_results_).loc[:,["params","mean_test_score"]])

#the best combination of parameters
print(grille.best_params_)

#the best performance
print(grille.best_score_)

#prediction with the best model
y_pred3 = grille.predict(X_test)

#accuracy rate
print(metrics.accuracy_score(y_test,y_pred3))

#### ATTRIBUTE SELECTION ####

#Logistic Regression class
from sklearn.linear_model import LogisticRegression

#instantiate an object
lr = LogisticRegression()

#RFE feature selection algorithm
from sklearn.feature_selection import RFE
selecteur = RFE(estimator=lr)

#launch the search
sol = selecteur.fit(X_app,y_app)

#number of selected attributes
print(sol.n_features_)

#selected attributes
print(sol.support_)

#order of delection
print(sol.ranking_)

#training set with the selected attributes
X_new_app = X_app[:,sol.support_]
print(X_new_app.shape)

#fit the model
modele_sel = lr.fit(X_new_app,y_app)

#test set with the selected variables
X_new_test = X_test[:,sol.support_]
print(X_new_test.shape)

#prediction
y_pred_sel = modele_sel.predict(X_new_test)

#evaluation - accuracy rate
print(metrics.accuracy_score(y_test,y_pred_sel))